<?php declare(strict_types=1);

namespace EuKit\Base\Services;

use EuKit\Base\Interfaces\ThemeServiceStartup;
use EuKit\Base\SiteLayout;
use Infomaker\Everyware\Base\Sidebars;

class SidebarServiceStartup implements ThemeServiceStartup
{
    public const CONTENT_SIZE_PREFIX = 'sidebar_';

    /**
     * @var array
     */
    private $registeredSidebars;

    public function registerSidebar(string $id, array $options): void
    {
        $this->registeredSidebars[$id] = $options;
    }

    public function registerSidebars(array $sidebars): void
    {
        foreach ($sidebars as $id => $options) {
            $this->registerSidebar($id, $options);
        }
    }

    public function setup(): void
    {
        $this->registerSidebars([
            // Article Page sidebars
            'article-page-top' => [
                'name' => __('Article page top', 'ewkit'),
                'description' => __('Container at the top of an article page.', 'ewkit'),
                'before_widget' => '<div class="article-page-top__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'article-page-content-above' => [
                'name' => __('Article page content above', 'ewkit'),
                'description' => __('Container above the article content on an article page.', 'ewkit'),
                'before_widget' => '<div class="article-page-content-above__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'sm' => 12,
                    'md' => 8,
                    'lg' => 9,
                ],
            ],
            'article-page-content-below' => [
                'name' => __('Article page content below', 'ewkit'),
                'description' => __('Container below the article content on an article page.', 'ewkit'),
                'before_widget' => '<div class="article-page-content-below__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                    'md' => 8,
                    'lg' => 9,
                ],
            ],
            'ew-below-article' => [
                'name' => __('Article below (DEPRECATED)', 'ewkit'),
                'description' => __('Use sidebar "Article page bottom" instead.', 'ewkit'),
                'before_widget' => '<div class="below-article-container__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'article-page-bottom' => [
                'name' => __('Article page bottom', 'ewkit'),
                'description' => __('Container at the bottom of an article page.', 'ewkit'),
                'before_widget' => '<div class="article-page-bottom__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'article-page-sidebar' => [
                'name' => __('Article page sidebar', 'ewkit'),
                'description' => __('Container on the side of an article page.', 'ewkit'),
                'before_widget' => '<div class="article-page-sidebar__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'md' => 4,
                    'lg' => 3,
                ],
            ],

            // Concept Page sidebars
            'concept-page-top' => [
                'name' => __('Concept page top', 'ewkit'),
                'description' => __('Container at the top of a concept page.', 'ewkit'),
                'before_widget' => '<div class="concept-page-top__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'concept-page-content-above' => [
                'name' => __('Concept page content above', 'ewkit'),
                'description' => __('Container above the content of a concept page.', 'ewkit'),
                'before_widget' => '<div class="concept-page-content-above__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'sm' => 12,
                    'lg' => 9,
                ],
            ],
            'concept-page-content-below' => [
                'name' => __('Concept page content below', 'ewkit'),
                'description' => __('Container below the content of a concept page.', 'ewkit'),
                'before_widget' => '<div class="concept-page-content-below__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                    'lg' => 9,
                ],
            ],
            'concept-page-bottom' => [
                'name' => __('Concept page bottom', 'ewkit'),
                'description' => __('Container at the bottom of a concept page.', 'ewkit'),
                'before_widget' => '<div class="concept-page-bottom__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'concept-page-sidebar' => [
                'name' => __('Concept page sidebar', 'ewkit'),
                'description' => __('Container on the side of a concept page.', 'ewkit'),
                'before_widget' => '<div class="concept-page-sidebar__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'md' => 4,
                    'lg' => 3,
                ],
            ],

            // Static Page sidebars
            'static-page-top' => [
                'name' => __('Static page top', 'ewkit'),
                'description' => __('Container at the top of a static page.', 'ewkit'),
                'before_widget' => '<div class="static-page-top__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'static-page-content-above' => [
                'name' => __('Static page content above', 'ewkit'),
                'description' => __('Container above the content of a static page.', 'ewkit'),
                'before_widget' => '<div class="static-page-content-above__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'sm' => 12,
                    'lg' => 9,
                ],
            ],
            'static-page-content-below' => [
                'name' => __('Static page content below', 'ewkit'),
                'description' => __('Container below the content of a static page.', 'ewkit'),
                'before_widget' => '<div class="static-page-content-below__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                    'lg' => 9,
                ],
            ],
            'static-page-bottom' => [
                'name' => __('Static page bottom', 'ewkit'),
                'description' => __('Container at the bottom of a static page.', 'ewkit'),
                'before_widget' => '<div class="static-page-bottom__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
            'static-page-sidebar' => [
                'name' => __('Static page sidebar', 'ewkit'),
                'description' => __('Container on the side of a static page.', 'ewkit'),
                'before_widget' => '<div class="static-page-sidebar__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'md' => 4,
                    'lg' => 3,
                ],
            ],

            'ew-footer' => [
                'name' => __('Footer', 'ewkit'),
                'description' => __('Container that is rendered just before the page footer.', 'ewkit'),
                'before_widget' => '<div class="footer-container__widget">',
                'after_widget' => '</div>',
                'content_sizes' => [
                    'xs' => 12,
                ],
            ],
        ]);

        foreach ($this->registeredSidebars as $id => $sidebar) {

            // Register content sizes separately.
            if (isset($sidebar['content_sizes'])) {
                SiteLayout::setContentSizes(self::CONTENT_SIZE_PREFIX . $id, $sidebar['content_sizes']);
                unset($sidebar['content_sizes']);
            }

            Sidebars::register($id, $sidebar);
        }

        add_filter('dynamic_sidebar_params', [self::class, 'applyContentSizes']);
    }

    /**
     * Filter for WP hook 'dynamic_sidebar_params'
     *
     * @param array $params
     *
     * @return array
     */
    public static function applyContentSizes(array $params): array
    {
        $contentSizes = SiteLayout::getContentSizes(self::CONTENT_SIZE_PREFIX . $params[0]['id']);
        if (!empty($contentSizes)) {
            $params[0]['class'] = trim(
                $params[0]['class'] . ' ' .
                implode(' ', SiteLayout::getContentSizeClasses($contentSizes))
            );
            $params[0]['template_args'] = [
                'max_cols' => SiteLayout::getContentSizeGridMaxColumns(),
                'content_sizes' => $contentSizes,
            ];
        }

       return $params;
    }
}
